﻿namespace Dienstfahrten.Logging
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    //using ArvatoSystems.Utility.Extensions;
    using Microsoft.SharePoint.Administration;

    // http://msdn.microsoft.com/en-us/library/gg512103.aspx

    /// <summary>
    /// Stellt Funktionen zum Hinterlegen von Nachrichten im SharePoint-Log bereit.
    /// </summary>
    internal sealed class Log : SPDiagnosticsServiceBase
    {
        private const string INFO = "Info";
        private const string WARNING = "Warning";
        private const string ERROR = "Error";
        private const string DEBUG = "Verbose";
        private const uint DEFAULT_ID = 0u;

        private const string PRODUCTNAME = "Insight News Ticker";
        public const string SERVICENAME = "Insight News Ticker Logging Service";

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse <see cref="Log"/>.
        /// </summary>
        public Log()
            : base(SERVICENAME, SPFarm.Local)
        {

        }

        /// <summary>
        /// Hinterlegt eine informative Nachricht im SharePoint-Log.
        /// </summary>
        /// <param name="message">Die Nachricht, die hinterlegt werden soll.</param>
        public void Info(string message)
        {
            this.Write(Level.Info, DEFAULT_ID, message);
        }

        /// <summary>
        /// Hinterlegt eine informative, formatierte Nachricht im SharePoint-Log.
        /// </summary>
        /// <param name="format">Die mit Formatierungsplatzhaltern versehende Nachricht, die
        /// hinterlegt werden soll.</param>
        /// <param name="args">Die Informationen, mit denen die Formatierungsplatzhalter gefüllt
        /// werden sollen.</param>
        public void Info(string format, params object[] args)
        {
            this.Write(
                Level.Info,
                DEFAULT_ID,
                string.Format(CultureInfo.CurrentCulture, format, args));
        }

        /// <summary>
        /// Hinterlegt eine Nachricht mit warnendem Inhalt im SharePoint-Log.
        /// </summary>
        /// <param name="message">Die Nachricht, die hinterlegt werden soll.</param>
        public void Warning(string message)
        {
            this.Write(Level.Warning, DEFAULT_ID, message);
        }

        /// <summary>
        /// Hinterlegt eine Nachricht mit formartiertem, warnendem Inhalt im SharePoint-Log.
        /// </summary>
        /// <param name="format">Die mit Formatierungsplatzhaltern versehende Nachricht, die
        /// hinterlegt werden soll.</param>
        /// <param name="args">Die Informationen, mit denen die Formatierungsplatzhalter gefüllt
        /// werden sollen.</param>
        public void Warning(string format, params object[] args)
        {
            this.Write(
                Level.Warning,
                DEFAULT_ID,
                string.Format(CultureInfo.CurrentCulture, format, args));
        }

        /// <summary>
        /// Hinterlegt eine Nachricht bezüglich eines Fehlers im SharePoint-Log.
        /// </summary>
        /// <param name="message">Die Nachricht, die hinterlegt werden soll.</param>
        public void Error(string message)
        {
            this.Write(Level.Error, DEFAULT_ID, message);
        }

        /// <summary>
        /// Hinterlegt eine Nachricht bezüglich eines Fehlers im SharePoint-Log.
        /// </summary>
        /// <param name="exception">Die Ausnahme, deren Inhalt als Nachricht hinterlegt
        /// werden soll.</param>
        public void Error(Exception exception)
        {
            if (exception != null)
            {
                this.Write(Level.Error, DEFAULT_ID, exception.ToString());
            }
        }

        /// <summary>
        /// Hinterlegt eine formatierte Nachricht bezüglich eines Fehlers im SharePoint-Log.
        /// </summary>
        /// <param name="format">Die mit Formatierungsplatzhaltern versehende Nachricht, die
        /// hinterlegt werden soll.</param>
        /// <param name="args">Die Informationen, mit denen die Formatierungsplatzhalter gefüllt
        /// werden sollen.</param>
        public void Error(string format, params object[] args)
        {
            this.Write(
                Level.Error,
                DEFAULT_ID,
                string.Format(CultureInfo.CurrentCulture, format, args));
        }

        /// <summary>
        /// Hinterlegt eine informative Nachricht für Entwickler im SharePoint-Log.
        /// </summary>
        /// <param name="message">Die Nachricht, die hinterlegt werden soll.</param>
        public void Debug(string message)
        {
            this.Write(Level.Verbose, DEFAULT_ID, message);
        }

        /// <summary>
        /// Hinterlegt eine formatierte, informative Nachricht für Entwickler im SharePoint-Log.
        /// </summary>
        /// <param name="format">Die mit Formatierungsplatzhaltern versehende Nachricht, die
        /// hinterlegt werden soll.</param>
        /// <param name="args">Die Informationen, mit denen die Formatierungsplatzhalter gefüllt
        /// werden sollen.</param>
        public void Debug(string format, params object[] args)
        {
            this.Write(
                Level.Verbose,
                DEFAULT_ID,
                string.Format(CultureInfo.CurrentCulture, format, args));
        }

        /// <summary>
        /// Hinterlegt eine Nachricht im SharePoint-Log und verwendet dabei die übergebene Stufe.
        /// </summary>
        /// <param name="level">Die Stufe, mit der die Nachricht hinterlegt werden soll.</param>
        /// <param name="id">Die ID, mit der die Nachricht hinterlegt werden soll.</param>
        /// <param name="message">Die Nachricht, die hinterlegt werden soll.</param>
        /// <exception cref="System.NotSupportedException">Die unter <paramref name="level"/>
        /// festgelegte Stufe wird nicht unterstützt.</exception>
        public void Write(Level level, uint id, string message)
        {
            //id.GreaterEqualThan(0u, "id");
            //message.NotNullOrEmpty("message");

            // Stufe bestimmen.
            SPDiagnosticsCategory category;
            switch (level)
            {
                case Level.Info:
                    category = this.Areas[PRODUCTNAME].Categories[INFO];
                    break;

                case Level.Warning:
                    category = this.Areas[PRODUCTNAME].Categories[WARNING];
                    break;

                case Level.Error:
                    category = this.Areas[PRODUCTNAME].Categories[ERROR];
                    break;

                case Level.Verbose:
                    category = this.Areas[PRODUCTNAME].Categories[DEBUG];
                    break;

                default:
                    throw new NotSupportedException(
                        string.Concat("Level \"", level.ToString(), "\" is not supported."));
            }

            // Nachricht im Log hinterlegen.
            this.WriteTrace(id, category, category.TraceSeverity, message, null);
        }

        /// <summary>
        /// Gibt die Diagnosekategorien zurück, die von diesem Log-Dienst verwendet werden.
        /// </summary>
        /// <returns>Eine Liste der verwendeten Diagnosekategorien.</returns>
        protected override IEnumerable<SPDiagnosticsArea> ProvideAreas()
        {
            return
                new SPDiagnosticsArea[] 
                {
                    new SPDiagnosticsArea(
                        PRODUCTNAME,
                        new SPDiagnosticsCategory[] {
                            new SPDiagnosticsCategory(INFO, TraceSeverity.Verbose, EventSeverity.Information), 
                            new SPDiagnosticsCategory(WARNING, TraceSeverity.Monitorable, EventSeverity.Warning),
                            new SPDiagnosticsCategory(ERROR, TraceSeverity.Unexpected, EventSeverity.Warning), 
                            new SPDiagnosticsCategory(DEBUG, TraceSeverity.Verbose, EventSeverity.Verbose)
                        })
                };
        }
    }
}
